unit AddLoginDlg;

interface

uses
  Winapi.Windows,
  Winapi.Messages,
  System.SysUtils,
  System.StrUtils,
  System.Variants,
  System.Classes,
  Vcl.Graphics,
  Vcl.Controls,
  Vcl.Forms,
  Vcl.Dialogs,
  Vcl.StdCtrls,

  VaxServerCOM,
  VaxTeleServerSIP,
  Codec;

type TAddLoginForm = class(TForm)

    // component instances

    Label1: TLabel;
    ComboLogin: TComboBox;
    Label2: TLabel;
    btnAddLogin: TButton;
    btnRemoveLogin: TButton;
    btnClear: TButton;
    GroupBoxCodecs: TGroupBox;
    GroupBoxCallRouting: TGroupBox;
    CheckBoxBlockAccount: TCheckBox;
    Label4: TLabel;
    ListBoxEnableCodecs: TListBox;
    btnDisableCodec: TButton;
    btnEnableCodec: TButton;
    Label5: TLabel;
    ListBoxDisableCodecs: TListBox;
    Label6: TLabel;
    EditCallPrefix: TEdit;
    Label7: TLabel;
    EditDialPrefix: TEdit;
    Label8: TLabel;
    ComboLines: TComboBox;
    EditPassword: TEdit;
    Label9: TLabel;
    Label3: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;



    procedure AddLogin_Click(Sender: TObject);
    procedure RemoveLogin_Click(Sender: TObject);
    procedure ClearLoginDlg_Click(Sender: TObject);
    procedure DisableCodec_Click(Sender: TObject);
    procedure EnableCodec_Click(Sender: TObject);
    procedure OnCreate(Sender: TObject);
    procedure ComboBox_SelIndexChanged(Sender: TObject);

    public

    constructor Create(); overload;
    constructor Create(AOwner : TComponent; var pVaxTeleSIP : TVaxTeleServerSIP); reintroduce; overload;


    // simple functions

  private

    procedure ShowUsers();
    procedure ShowLines();
    procedure ShowDirectProxies();

    function AddLoginChecks() : boolean;
    procedure ClearDlgContent();
    procedure LoadCodecs();
    procedure UpdateDlgContent(sLoginSIP, sPassword, sCodecList, sDialPrefix, sUserPrefix, sToPeerName : string; bBlock : Boolean);


  public

    procedure OnSetActive();

  end;

var
  AddLoginForm: TAddLoginForm;
  m_objCodec : TCodec;
  m_pVaxTeleSIP : TVaxTeleServerSIP;

implementation

{$R *.dfm}

    constructor TAddLoginForm.Create();
    begin

    end;

    constructor TAddLoginForm.Create(AOwner : TComponent; var pVaxTeleSIP : TVaxTeleServerSIP);
    begin
        inherited Create(AOwner);
        m_pVaxTeleSIP := pVaxTeleSIP;
    end;


// functions

function TAddLoginForm.AddLoginChecks() : boolean;

var
nIndex, nCount : integer;
sDigits, sLoginName : string;
sChar : char;

begin

    // Check if login name is enered.

    if(Length(ComboLogin.Text)  = 0) then
    begin

      ShowMessage('Please enter login');
      Result := false;
    end;

    // check if the login name consists of numbers only

    sDigits := '0123456789';
    sLoginName := ComboLogin.Text;
    nCount := 0;

    for nIndex := 0 to Length(sLoginName) do
    begin

        if( AnsiPos(sLoginName[nIndex + 1], sDigits) = 0) then
        begin
            ShowMessage('Login should contain only numerals 0 through 9. For example: 12, 324323, 800122, 2109756 etc.');
            Result := false;
            exit;
        end;

        nCount := nCount + 1;

        if(nCount >= Length(sLoginName)) then
        begin
            Result := true;
            break;
        end;


    end;

    if(Length(EditPassword.Text)  = 0) then
    begin

      ShowMessage('Please enter password');
      Result := false;
      exit;
    end;

    if(Length(sLoginName) > 20) then
    begin
        ShowMessage('Login length should be less than 20');
        Result := false;
        exit;
    end;

    if(Length(EditPassword.Text ) > 20) then
    begin
      ShowMessage('Password length should be less than 20.');
      Result := false;
      exit;
    end;

    if( AnsiLeftStr(sLoginName, 2) = '00') then
    begin
        ShowMessage('Please do not enter 00 as login prefix. 00 prefix is use to prepare dial plan.');
        Result := false;
        exit;
    end;

    if(ListBoxEnableCodecs.Items.Count <= 0) then
    begin
        ShowMessage('Please select a codec.');
        Result := false;
        exit;
    end;

end;

procedure TAddLoginForm.ClearDlgContent();
begin

    ComboLogin.Text := '';
    EditPassword.Text := '';

    ListBoxEnableCodecs.Items.Clear();
    ListBoxDisableCodecs.Items.Clear();

    CheckBoxBlockAccount.Checked := false;

    LoadCodecs();

end;

procedure TAddLoginForm.LoadCodecs();

var
  nIndex : Integer;

begin

    for nIndex := 0 to m_objCodec.TOTAL_SIZE_CODEC - 1  do
    begin
        ListBoxEnableCodecs.Items.Add(m_objCodec.GetCodecName(IntToStr(nIndex)));
    end;

end;

procedure TAddLoginForm.UpdateDlgContent(sLoginSIP, sPassword, sCodecList, sDialPrefix, sUserPrefix, sToPeerName : string; bBlock : Boolean);

var
nIndex : integer;
sCodecs, sCodecName : string;

begin

    EditPassword.Text := sPassword;
    EditDialPrefix.Text := sDialPrefix;
    EditCallPrefix.Text := sUserPrefix;

    ListBoxEnableCodecs.Items.Clear();
    ListBoxDisableCodecs.Items.Clear();

    sCodecs := '';
    for nIndex := 0 to m_objCodec.TOTAL_SIZE_CODEC - 1 do
        ListBoxDisableCodecs.Items.Add(m_objCodec.GetCodecName(IntToStr(nIndex)));

    for nIndex := 1 to Length(sCodecList) do
    begin
        sCodecName := m_objCodec.GetCodecName(sCodecList[nIndex]);    // get codec name
        ListBoxEnableCodecs.Items.Add(sCodecName);
        ListBoxDisableCodecs.Items.Delete(ListBoxDisableCodecs.Items.IndexOf(sCodecName));

    end;


end;

procedure TAddLoginForm.ShowUsers();

var
nIndex : integer;
objListUsers : TStringList;

////////////////////
sLogin, sPassword, sUserPrefix : string ;
sDialPrefix , sCodecList, sToPeerName : string;
nToPeerType : integer;
bBlock : boolean;

begin
    objListUsers := TStringList.Create();
    m_pVaxTeleSIP.GetUserAll(objListUsers);

    if(objListUsers.Count <= 0) then
        exit;

    for nIndex := 0 to objListUsers.Count - 1 do
        ComboLogin.Items.Add(objListUsers[nIndex]);

    ComboLogin.ItemIndex := 0;

    sLogin := ComboLogin.Text;

    m_pVaxTeleSIP.GetUserDetails(sLogin, sPassword, sCodecList, bBlock, sUserPrefix, sDialPrefix, nToPeerType, sToPeerName);
    UpdateDlgContent(sLogin, sPassword, sCodecList, sDialPrefix, sUserPrefix, sToPeerName, bBlock);

end;
procedure TAddLoginForm.ShowLines();

var
    objListLines : TStringList;
    nIndex: Integer;
    sLineName : string;
begin

    ComboLines.Clear;

    objListLines := TStringList.Create();
    //m_pVaxTeleSIP.GetLineAll(objListLines);

    if(objListLines.Count <= 0) then
        exit;

    for nIndex := 0 to objListLines.Count - 1 do
    begin
        sLineName := objListLines[nIndex];
        ComboLines.Items.AddObject(sLineName, TObject(TVaxServerCOM.VAX_PEER_TYPE_LINE));
    end;

    if(ComboLines.Items.Count > 0) then
        ComboLines.ItemIndex := 0;

end;
procedure TAddLoginForm.ShowDirectProxies();

var
    objListProxies : TStringList;
    nIndex: Integer;

begin

    objListProxies := TStringList.Create();
    m_pVaxTeleSIP.GetDirectProxyAll(objListProxies);

    if(objListProxies.Count <= 0) then
        exit;

    for nIndex := 0 to objListProxies.Count - 1 do
        ComboLines.Items.AddObject(objListProxies[nIndex], TObject(TVaxServerCOM.VAX_PEER_TYPE_DIRECT_PROXY));

end;


procedure TAddLoginForm.OnSetActive();
begin

    ComboLogin.Items.Clear();

    ShowLines();
    ShowDirectProxies();
    ShowUsers();

end;

// Events

procedure TAddLoginForm.AddLogin_Click(Sender: TObject);

var
  nIndex, nRouteType: Integer;
  sCodecList, sCodec, sLoginName, sCodecName, sCodecNo, sRouteName : string;

begin

    if(AddLoginChecks() = false) then
        exit;

    // Get the codec list

    sCodecList := '';
    sLoginName := '';

    for nIndex := 0 to ListBoxEnableCodecs.Items.Count - 1 do
    begin
        sCodecName := ListBoxEnableCodecs.Items[nIndex];    // get codec name
        sCodecNo := m_objCodec.GetCodecNo(sCodecName);  // use codec name and get codec no
        sCodecList := sCodecList + sCodecNo;     // concat all these codec numbers
    end;

    nRouteType := TVaxServerCOM.VAX_PEER_TYPE_USER; // see above note.

    if(ComboLines.Items.Count > 0) then
    begin
        nRouteType := Integer(ComboLines.Items.Objects[ComboLines.Items.IndexOf(ComboLines.Text)]);
        sRouteName := ComboLines.Items[ComboLines.ItemIndex];
    end;

    m_pVaxTeleSIP.RemoveUser(ComboLogin.Text);

   if (m_pVaxTeleSIP.AddUser(ComboLogin.Text, EditPassword.Text, sCodecList, CheckBoxBlockAccount.Checked, EditCallPrefix.Text,
                                EditDialPrefix.Text, nRouteType, sRouteName) = false) then
    begin
        ShowMessage('Login could not be created.');
        Exit;
    end;

    ComboLogin.Items.Add(ComboLogin.Text );
    ShowMessage('Login added successfully.');

end;

procedure TAddLoginForm.RemoveLogin_Click(Sender: TObject);

var
sLoginName : string;
nIndex : integer;

begin

    if(Length(ComboLogin.Items[ComboLogin.ItemIndex] ) = 0) then
    begin
        ShowMessage('Please enter a login.');
        exit;
    end;

    m_pVaxTeleSIP.RemoveUser(ComboLogin.Items[ComboLogin.ItemIndex]);

    ComboLogin.Items.Delete(ComboLogin.Items.IndexOf(ComboLogin.Text));
    ShowMessage('Login removed successfully.');
    ClearDlgContent();

end;

procedure TAddLoginForm.ClearLoginDlg_Click(Sender: TObject);
begin
    ClearDlgContent();
end;

procedure TAddLoginForm.ComboBox_SelIndexChanged(Sender: TObject);
var
sLoginName, sPassword, sUserPrefix : string ;
sDialPrefix , sCodecList, sToPeerName : string;
nToPeerType : integer;
bBlock : boolean;

begin

    sPassword := '';
    sUserPrefix := '';
    sDialPrefix := '';
    sCodecList := '01234';
    sToPeerName := '';
    nToPeerType := -1;
    bBlock := false;

    sLoginName := ComboLogin.Text;
    m_pVaxTeleSIP.GetUserDetails(sLoginName, sPassword, sCodecList, bBlock, sUserPrefix, sDialPrefix, nToPeerType, sToPeerName);

    UpdateDlgContent(sLoginName, sPassword, sCodecList, sDialPrefix, sUserPrefix, sToPeerName, bBlock);


end;

procedure TAddLoginForm.DisableCodec_Click(Sender: TObject);
begin

    if(ListBoxEnableCodecs.Items.Count <= 0) then
        exit;

    if(ListBoxEnableCodecs.ItemIndex = -1) then
    begin
        ShowMessage('Please choose a codec first');
        exit;
    end;

    ListBoxDisableCodecs.Items.Add(ListBoxEnableCodecs.Items[ListBoxEnableCodecs.ItemIndex]);
    ListBoxEnableCodecs.Items.Delete(ListBoxEnableCodecs.ItemIndex);

end;

procedure TAddLoginForm.EnableCodec_Click(Sender: TObject);
begin

    if(ListBoxDisableCodecs.Items.Count <= 0) then
        exit;

    if(ListBoxDisableCodecs.ItemIndex = -1) then
    begin
        ShowMessage('Please choose a codec first');
        exit;
    end;

    ListBoxEnableCodecs.Items.Add(ListBoxDisableCodecs.Items[ListBoxDisableCodecs.ItemIndex]);
    ListBoxDisableCodecs.Items.Delete(ListBoxDisableCodecs.ItemIndex);

end;

procedure TAddLoginForm.OnCreate(Sender: TObject);
begin
    LoadCodecs();
end;




end.


