unit VaxRingGroupAgentList;

interface

uses
    System.SysUtils,
    System.Classes,
    Generics.Collections;

type TVaxAgentDetails = class

    public
        m_sAgentName : string;
end;

type TVaxAgentList = class

    m_objRingGroupAgentList : TObjectDictionary<string, TVaxAgentDetails>;

 private

        function GenKey(sValue : string) : string;
        procedure DeleteAgentAll();

        function FindAgent(sAgentName : string) : TVaxAgentDetails;

 public

        constructor Create();
        Destructor Destroy();

        function AddAgent(sAgentName : string) : boolean;
        procedure RemoveAgent(sAgentName : string);

        function IsAgentExisted(sAgentName : string) : Boolean;

        function GetAllAgentName(var objAgentList : TStringList) : boolean;

        function GetAgentCount() : Integer;

end;
implementation

        constructor TVaxAgentList.Create();
        begin
            m_objRingGroupAgentList := TObjectDictionary<string, TVaxAgentDetails>.Create([doOwnsValues], 256);
        end;

        Destructor TVaxAgentList.Destroy();
        begin

            DeleteAgentAll();
            m_objRingGroupAgentList.Free;

        end;

        function TVaxAgentList.GenKey(sValue : string) : string;
        begin

            Result := AnsiLowerCase(sValue);

        end;

        procedure TVaxAgentList.DeleteAgentAll();
        begin

            m_objRingGroupAgentList.Clear();

        end;

        function TVaxAgentList.FindAgent(sAgentName : string) : TVaxAgentDetails;
        var
            objAgentDetails : TVaxAgentDetails;
            bResult : Boolean;
        begin
            objAgentDetails := Nil;

            bResult := m_objRingGroupAgentList.TryGetValue(sAgentName, objAgentDetails);
            if(bResult = False) then
                Exit(nil);

            Result := objAgentDetails;
        end;

        function TVaxAgentList.AddAgent(sAgentName : string) : boolean;
         var
            sKey : string;
            objAgentDetails : TVaxAgentDetails;

        begin
            objAgentDetails :=  FindAgent(sAgentName);

            if(objAgentDetails <> Nil) then
                Exit(False);

            objAgentDetails := TVaxAgentDetails.Create();

            objAgentDetails.m_sAgentName := sAgentName;
            m_objRingGroupAgentList.Add(sAgentName, objAgentDetails);

            Exit(True);
        end;


        procedure TVaxAgentList.RemoveAgent(sAgentName : string);
         var
            sKey : string;
            objAgentDetails : TVaxAgentDetails;

        begin
            objAgentDetails :=  FindAgent(sAgentName);

            if(objAgentDetails <> Nil) then
                Exit();

             m_objRingGroupAgentList.Remove(AnsiLowerCase(sAgentName));

        end;

        function TVaxAgentList.IsAgentExisted(sAgentName : string) : Boolean;
        var
            sKey : string;
            objAgentDetails : TVaxAgentDetails;

        begin
            objAgentDetails :=  FindAgent(sAgentName);

            if(objAgentDetails <> Nil) then
                Exit(True);

            Exit(False);

        end;

        function TVaxAgentList.GetAllAgentName(var objAgentList : TStringList) : boolean;
        var
            objAgentDetails : TVaxAgentDetails;
            nIndex : Integer;
        begin

            if(m_objRingGroupAgentList.Count <= 0) then
                Exit(False);

            for objAgentDetails in m_objRingGroupAgentList.Values do
            begin
                objAgentList.Add(objAgentDetails.m_sAgentName);
            end;

            Result := True;
        end;

        function TVaxAgentList.GetAgentCount() : Integer;
        begin
            Result := m_objRingGroupAgentList.Count;
        end;





end.
