// CallPickUpDlg.cpp : implementation file
//

#include "stdafx.h"
#include "afxDialogEx.h"
#include "VaxTeleServerApp.h"
#include "CallPickUpDlg.h"
#include "VaxServerSIP.h"

// CCallPickUpDlg dialog

IMPLEMENT_DYNAMIC(CCallPickUpDlg, CPropertyPage)

CCallPickUpDlg::CCallPickUpDlg(CVaxServerSIP* pVaxTeleSIP): CPropertyPage(CCallPickUpDlg::IDD)
{
	m_pVaxTeleSIP = pVaxTeleSIP;
	m_sPickUpGroupName = _T("");
	m_sPickUpMemberName = _T("");
	m_sPickUpLoginName = _T("");
}

CCallPickUpDlg::~CCallPickUpDlg()
{
}

void CCallPickUpDlg::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_COMBO_CALL_PICKUP_GROUP, m_ctrlComboPickUpGroup);
	DDX_CBString(pDX, IDC_COMBO_CALL_PICKUP_GROUP, m_sPickUpGroupName);
	DDX_Control(pDX, IDC_LIST_PICKUP_MEMBER, m_ctrlListPickUpMember);
	DDX_Control(pDX, IDC_COMBO_LOGIN_PICKUP, m_ctrlListLogin);
	DDX_CBString(pDX, IDC_COMBO_LOGIN_PICKUP, m_sPickUpLoginName);
}


BEGIN_MESSAGE_MAP(CCallPickUpDlg, CPropertyPage)
	ON_BN_CLICKED(IDC_BUTTON_ADD_CALL_PICKUP_GROUP, &CCallPickUpDlg::OnButtonCallPickUpGroupAdd)
	ON_BN_CLICKED(IDC_BUTTON_REMOVE_CALL_PICKUP_GROUP, &CCallPickUpDlg::OnButtonCallPickUpGroupRemove)
	ON_BN_CLICKED(IDC_BUTTON_CLEAR_CALLPICKUP, &CCallPickUpDlg::OnButtonClearCallPickUpDlg)
	ON_BN_CLICKED(IDC_BUTTON_ADD_CALLPICKUP_MEMBER, &CCallPickUpDlg::OnButtonCallPickUpMemberAdd)
	ON_BN_CLICKED(IDC_BUTTON_REMOVE_CALLPICKUP_MEMBER, &CCallPickUpDlg::OnButtonCallPickUpMemberRemove)
	ON_BN_CLICKED(IDC_BUTTON_CLEAR_CALLPICKUP, &CCallPickUpDlg::OnButtonClearCallPickUpGroupDlg)
	ON_BN_CLICKED(IDC_BUTTON_CLEAR_CALLPICKUP_MEMBER, &CCallPickUpDlg::OnButtonClearCallPickUpMemberDlg)
	ON_CBN_SELCHANGE(IDC_COMBO_CALL_PICKUP_GROUP, &CCallPickUpDlg::OnCbnSelchangeComboCallPickupGroup)
END_MESSAGE_MAP()


BOOL CCallPickUpDlg::OnSetActive()
{
	AddPickUpGroups();
	AddUsers();

	ShowPickUpGroupDetails();
	
	return CPropertyPage::OnSetActive();
}

void CCallPickUpDlg::OnButtonCallPickUpGroupAdd()
{
	UpdateData();

	if(!AddPickupGroupChecks())
		return;

	if(m_ctrlComboPickUpGroup.FindStringExact(0, m_sPickUpGroupName) != -1)
	{
		if(MessageBox("Pickup Group already exists, do you want to continue?", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_YESNO) == IDNO)
			return;
	}

	m_pVaxTeleSIP->RemoveCallPickUpGroup(m_sPickUpGroupName);

	if(!m_pVaxTeleSIP->AddCallPickUpGroup(m_sPickUpGroupName))
	{
		MessageBox("PickUp Group could't be added.", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_OK);
		return;
	}

	CString sPickUpMember;
	for(int nIndex = 0; nIndex < m_ctrlListPickUpMember.GetCount(); nIndex++)
	{
		m_ctrlListPickUpMember.GetText(nIndex, sPickUpMember);
		m_pVaxTeleSIP->AddCallPickUpGroupMember(m_sPickUpGroupName, sPickUpMember);
	}

	if(m_ctrlComboPickUpGroup.FindString(0, m_sPickUpGroupName) != -1)
		m_ctrlComboPickUpGroup.AddString(m_sPickUpGroupName);
			
	m_ctrlComboPickUpGroup.SetCurSel(0);
	m_ctrlComboPickUpGroup.GetLBText(0, m_sPickUpGroupName);

	UpdateData(FALSE);

	MessageBox("PickUp Group created successfully", "Success", MB_ICONEXCLAMATION|MB_OK);
}

void CCallPickUpDlg::OnButtonCallPickUpGroupRemove()
{
	UpdateData();
	
	m_pVaxTeleSIP->RemoveCallPickUpGroup(m_sPickUpGroupName);

	int nIndex = m_ctrlComboPickUpGroup.FindStringExact(0, m_sPickUpGroupName);
		
	if(nIndex != -1)
		m_ctrlComboPickUpGroup.DeleteString(nIndex);

	MessageBox("PickUp Group removed successfully!", "Success", MB_ICONEXCLAMATION|MB_OK);

	ClearDlgContent();
}

void CCallPickUpDlg::OnButtonClearCallPickUpDlg()
{
	ClearDlgContent();
}

void CCallPickUpDlg::OnButtonCallPickUpMemberAdd()
{
	UpdateData();

	if(m_ctrlListPickUpMember.FindString(0, m_sPickUpLoginName) == -1)
		m_ctrlListPickUpMember.AddString(m_sPickUpLoginName);

	UpdateData(FALSE);
}

void CCallPickUpDlg::OnButtonCallPickUpMemberRemove()
{
	m_ctrlListPickUpMember.DeleteString(m_ctrlListPickUpMember.GetCurSel());

	UpdateData(FALSE);
}

void CCallPickUpDlg::OnButtonClearCallPickUpGroupDlg()
{
	m_ctrlListPickUpMember.ResetContent();
}

void CCallPickUpDlg::OnButtonClearCallPickUpMemberDlg()
{
	ClearAgentDlgContent();
}

BOOL CCallPickUpDlg::AddPickupGroupChecks()
{
	if(m_sPickUpGroupName.IsEmpty())
	{
		MessageBox("Please fill group name first.", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_OK);
		return FALSE;
	}

	return TRUE;
}

void CCallPickUpDlg::ClearDlgContent()
{
	UpdateData();

	m_sPickUpGroupName = "";
	ClearAgentDlgContent();
	
	UpdateData(FALSE);
}

void CCallPickUpDlg::ClearAgentDlgContent()
{
	m_ctrlListPickUpMember.ResetContent();

	UpdateData(FALSE);
}

void CCallPickUpDlg::AddUsers()
{
	m_ctrlListLogin.ResetContent();

	CStringList objListUsers;
	m_pVaxTeleSIP->GetUserAll(&objListUsers);

	while(!objListUsers.IsEmpty())
		m_ctrlListLogin.AddString(objListUsers.RemoveHead());
			
	if(m_ctrlListLogin.GetCount() > 0)
		m_ctrlListLogin.SetCurSel(0);
}

void CCallPickUpDlg::ShowPickUpGroupDetails()
{
	UpdateData();

	int nSelectedIndex = m_ctrlComboPickUpGroup.GetCurSel();
	if(nSelectedIndex == -1) return;

	CString sPickUpGroupName;
	m_ctrlComboPickUpGroup.GetLBText(nSelectedIndex, sPickUpGroupName);
	
	m_sPickUpGroupName = sPickUpGroupName;
	
	CStringList objListMember;
	
	if(!m_pVaxTeleSIP->GetPickUpMemberName(sPickUpGroupName, &objListMember))
		return;

	m_ctrlListPickUpMember.ResetContent();

	while(!objListMember.IsEmpty())
		m_ctrlListPickUpMember.AddString(objListMember.RemoveHead());

	UpdateData(FALSE);
}

void CCallPickUpDlg::AddPickUpGroups()
{
	UpdateData();

	m_ctrlComboPickUpGroup.ResetContent();
	
	CStringList slistPickUpGroup;
	m_pVaxTeleSIP->GetPickUpGroupNameAll(&slistPickUpGroup);

	while(!slistPickUpGroup.IsEmpty())
		m_ctrlComboPickUpGroup.AddString(slistPickUpGroup.RemoveHead());

	if(m_ctrlComboPickUpGroup.GetCount() > 0)
		m_ctrlComboPickUpGroup.SetCurSel(0);
}

void CCallPickUpDlg::OnCbnSelchangeComboCallPickupGroup()
{
	ShowPickUpGroupDetails();
}
