#include "StdAfx.h"
#include "Define.h"
#include "VaxServerExCOM.h"
#include "ConfRoom.h"
#include "VaxCallSession.h"
#include "VaxRegUser.h"
#include"VaxUserData.h"
#include "VaxChat.h"
#include "VaxLine.h"
#include "VaxDirectProxy.h"
#include "VaxCallSessionRoute.h"

#include "CallPickUpList.h"
#include "RingGroupList.h"
#include "QueueList.h"
#include "VaxCallSession.h"

#include"VaxRegUserList.h"

CVaxServerExCOM* CVaxServerExCOM::m_pVaxServerExCOM = NULL;

CVaxServerExCOM::CVaxServerExCOM()
{
	if(m_pVaxServerExCOM == NULL)
		m_pVaxServerExCOM = this;

	::InitializeCriticalSection(&m_lCriticalSection);

	m_lDialToneId = -1;
	m_lMusicCallParkId = -1;

	m_bInitilized = FALSE;
		
	m_pConfRoom = new CConfRoom(this);
	m_pChat = new CVaxChat(this);
	m_pRouteCallSIP = new CVaxCallSessionRoute(this);

	m_pQueueList = new CQueueList();
	m_pRingGroupList = new CRingGroupList();
	m_pCallPickUpList = new CCallPickUpList();

	m_pRegUserList = new CVaxRegUserList();
}

CVaxServerExCOM::~CVaxServerExCOM()
{
	UnInitialize();

	delete m_pConfRoom;
	delete m_pChat;
	delete m_pRouteCallSIP;
	delete m_pRegUserList;

	delete m_pQueueList;
	delete m_pRingGroupList;
	delete m_pCallPickUpList;
	
	CVaxServerCOM::UnLoadObjectCOM();
	::DeleteCriticalSection(&m_lCriticalSection);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::SetLicenseKey(LPCTSTR sLicenseKey)
{
	m_sLicenseKey = sLicenseKey;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

BOOL CVaxServerExCOM::Initialize(LPCTSTR sDomainRealm, LPCTSTR sSIPListenIP, UINT nSIPListenPort, LPCTSTR sRTPListenIP, UINT nRTPListenPort)
{
	CVaxServerCOM::SetLicenseKey(m_sLicenseKey);

	if(!CVaxServerCOM::Initialize(sDomainRealm, sSIPListenIP, nSIPListenPort, sRTPListenIP, nRTPListenPort))
		return FALSE;

	m_bInitilized = TRUE;
	OnInitialize();

	return TRUE;
}

void CVaxServerExCOM::UnInitialize()
{
	m_bInitilized = FALSE;

	::EnterCriticalSection(&m_lCriticalSection);
			
	m_pConfRoom->DeleteRoomAll();
	m_pRouteCallSIP->ResetOBJ();

	m_pCallPickUpList->DeletePickUpGroupAll();
	m_pQueueList->DeleteQueueAll();
	m_pRingGroupList->DeleteRingGroupAll();

	DeleteUserDataAll();
	DeleteCallSessionDataAll();
	DeleteLineDataAll();
	
	CVaxServerCOM::UnInitialize();

	::LeaveCriticalSection(&m_lCriticalSection);
}

BOOL CVaxServerExCOM::IsInitialized()
{

	return m_bInitilized;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

BOOL CVaxServerExCOM::AddUser(LPCTSTR sUserName, LPCTSTR sPassword, LPCTSTR sCodecList)
{
	::EnterCriticalSection(&m_lCriticalSection);
			
	CVaxUserData* pUserData = new CVaxUserData();
	pUserData->SetData(sPassword, sCodecList, false, "", "", -1, sUserName, false);
	
	CVaxServerCOM::AddUser(sUserName, sPassword, sCodecList);
	BOOL bResult = CVaxServerCOM::SetUserData(sUserName, pUserData);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

BOOL CVaxServerExCOM::AddUser(LPCTSTR sUserName, LPCTSTR sPassword, LPCTSTR sCodecList, BOOL bBlocked, LPCTSTR sUserPrefix, LPCTSTR sDialPrefix, int nToPeerType, LPCTSTR sToPeerName, BOOL bStealthlisten)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	CVaxUserData* pUser = new CVaxUserData();
	pUser->SetData(sPassword, sCodecList, bBlocked, sUserPrefix, sDialPrefix, nToPeerType, sToPeerName, bStealthlisten);

	CVaxServerCOM::AddUser(sUserName, sPassword, sCodecList);
	BOOL bResult = CVaxServerCOM::SetUserData(sUserName, pUser);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

void CVaxServerExCOM::RemoveUser(LPCTSTR sUserName)
{      
	::EnterCriticalSection(&m_lCriticalSection);
		
	if(!IsUserExist(sUserName))
	{
		::LeaveCriticalSection(&m_lCriticalSection);
		return;
	}
	 
	CVaxUserData* pUser = (CVaxUserData*) CVaxServerCOM::GetUserData(sUserName);
	delete pUser;
	
	CVaxServerCOM::RemoveUser(sUserName);
	::LeaveCriticalSection(&m_lCriticalSection);
}

BOOL CVaxServerExCOM::IsUserExist(LPCTSTR sUserName)
{
	CVaxUserData* pUser = NULL;

	TrvsUserStart();
	CString sUser = "";

	while(TRUE)
	{
		sUser = TrvsUserNext();
		if(sUser.IsEmpty())
		break;

		if(sUser.Compare(sUserName) == 0)
			return TRUE;
	}

	return FALSE;
}

void CVaxServerExCOM::GetUserData(LPCTSTR sUserName, CString& sPassword, CString& sCodecList, BOOL& bBlocked, CString& sUserPrefix, CString& sDialPrefix, int& nToPeerType, CString& sToPeerName, BOOL& bStealthlisten)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	if(!IsUserExist(sUserName))
	{
		::LeaveCriticalSection(&m_lCriticalSection);
		return;
	}

	CVaxUserData* pUser = (CVaxUserData*) CVaxServerCOM::GetUserData(sUserName);

	if(pUser == NULL)
	{
		::LeaveCriticalSection(&m_lCriticalSection);
		return;
	}

	pUser->GetData(sPassword, sCodecList,  bBlocked, sUserPrefix, sDialPrefix, nToPeerType, sToPeerName, bStealthlisten);
	::LeaveCriticalSection(&m_lCriticalSection);
}


//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

BOOL CVaxServerExCOM::AddLine(LPCTSTR sLineName, LPCTSTR sDisplayName, LPCTSTR sUserName, LPCTSTR sAuthUser, LPCTSTR sAuthPwd, LPCTSTR sDomainRealm, LPCTSTR sProxySIP, LPCTSTR sCodecList, BOOL bRegister, int nToPeerType, LPCTSTR sToPeerName, BOOL bRejected, LPCTSTR sRoomName, LPCTSTR sQueueName, LPCTSTR sRingGroupName)
{
	::EnterCriticalSection(&m_lCriticalSection);
			
	BOOL bResult = CVaxServerCOM::AddLine(sLineName, sDisplayName, sUserName, sAuthUser, sAuthPwd, sDomainRealm, sProxySIP, "", sCodecList);
	if(!bResult)
	{
		::LeaveCriticalSection(&m_lCriticalSection);
		return FALSE;
	}

	CVaxLine* pLine = new CVaxLine(this);
	
	CVaxServerCOM::SetLineData(sLineName, (HANDLE) pLine);
	pLine->SetData(sDisplayName, sUserName, sAuthUser, sAuthPwd, sDomainRealm, sProxySIP, sCodecList, bRegister, nToPeerType, sToPeerName, bRejected, sRoomName, sQueueName, sRingGroupName);

	if(bRegister)
		CVaxServerCOM::RegisterLine(sLineName, 1800);

	::LeaveCriticalSection(&m_lCriticalSection);
	return TRUE;
}

BOOL CVaxServerExCOM::IsLineExist(LPCTSTR sLineName)
{
	CVaxLine* pLine = NULL;

	TrvsLineStart();
	CString sLine;

	while(TRUE)
	{
		sLine = TrvsLineNext();
		
		if(sLine.IsEmpty())
			break;

		if(sLine == sLineName)
			return TRUE;
	}

	return FALSE;
}

void CVaxServerExCOM::RemoveLine(LPCTSTR sLineName)
{
	::EnterCriticalSection(&m_lCriticalSection);
		
	if(!IsLineExist(sLineName))
	{
		::LeaveCriticalSection(&m_lCriticalSection);
		return;
	}

	CVaxLine* pLine = (CVaxLine*) CVaxServerCOM::GetLineData(sLineName);
	delete pLine;

	CVaxServerCOM::UnRegisterLine(sLineName);
	CVaxServerCOM::RemoveLine(sLineName);

	::LeaveCriticalSection(&m_lCriticalSection);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

BOOL CVaxServerExCOM::AddDirectProxySIP(LPCTSTR sProxyName, LPCTSTR sAuthLogin, LPCTSTR sAuthPwd, LPCTSTR sDomainRealm, LPCTSTR sProxyIP, LONG lProxyPort, LPCTSTR sCodecList, int nToPeerType, LPCTSTR sToPeerName, BOOL bRejected, LPCTSTR sRoomName, LPCTSTR sQueueName, LPCTSTR sRingGroupName)
{
	::EnterCriticalSection(&m_lCriticalSection);
			
	BOOL bResult = CVaxServerCOM::AddDirectProxySIP(sProxyName, sAuthLogin, sAuthPwd, sDomainRealm, sProxyIP, lProxyPort, sCodecList);
	
	if(!bResult)
	{
		::LeaveCriticalSection(&m_lCriticalSection);
		return FALSE;
	}

	CVaxDirectProxy* pDirectProxy = new CVaxDirectProxy(this);
	
	CVaxServerCOM::SetDirectProxyData(sProxyName, (HANDLE) pDirectProxy);
	pDirectProxy->SetData(sAuthLogin, sAuthPwd, sDomainRealm, sProxyIP, lProxyPort, sCodecList, nToPeerType, sToPeerName, bRejected, sRoomName, sQueueName, sRingGroupName);
	
	::LeaveCriticalSection(&m_lCriticalSection);
	return TRUE;
}

BOOL CVaxServerExCOM::IsProxyExist(LPCTSTR sProxyName)
{
	TrvsDirectProxyStart();
	CString sProxy = "";

	while(TRUE)
	{
		sProxy = TrvsDirectProxyNext();
		if(sProxy.IsEmpty())
		break;

		if(sProxy.Compare(sProxyName) == 0)
			return TRUE;
	}

	return FALSE;
}

void CVaxServerExCOM::RemoveDirectProxySIP(LPCTSTR sProxyName)
{
	::EnterCriticalSection(&m_lCriticalSection);
			
	if(!IsProxyExist(sProxyName))
	{
		::LeaveCriticalSection(&m_lCriticalSection);
		return;
	}

	CVaxDirectProxy* pDirectProxy = (CVaxDirectProxy*) CVaxServerCOM::GetDirectProxyData(sProxyName);
	delete pDirectProxy;

	CVaxServerCOM::RemoveDirectProxySIP(sProxyName);
	::LeaveCriticalSection(&m_lCriticalSection);
}


//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

LONG CVaxServerExCOM::GetVaxErrorCode()
{
	::EnterCriticalSection(&m_lCriticalSection);
		
	LONG lErrorNo = CVaxServerCOM::GetVaxErrorCode();

	::LeaveCriticalSection(&m_lCriticalSection);
	return lErrorNo;
}

BOOL CVaxServerExCOM::DialToneToCallSession(BOOL bEnable, LONG lSessionId)
{
	::EnterCriticalSection(&m_lCriticalSection);
		
	if(m_lDialToneId == -1)
	{
		::LeaveCriticalSection(&m_lCriticalSection);
		return TRUE;
	}
	
	BOOL bResult = CVaxServerCOM::DialToneToCallSession(bEnable, lSessionId, m_lDialToneId);
	
	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

BOOL CVaxServerExCOM::AddRoom(LPCTSTR sRoomName, LPCTSTR sRoomNo)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	BOOL bResult = m_pConfRoom->AddRoom(sRoomName, sRoomNo);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

void CVaxServerExCOM::RemoveRoom(LPCTSTR sRoomName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	m_pConfRoom->RemoveRoom(sRoomName);
	::LeaveCriticalSection(&m_lCriticalSection);
}

BOOL CVaxServerExCOM::GetRoomDetails(LPCTSTR sRoomName, CString& sRoomNo)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	BOOL bResult = m_pConfRoom->GetRoomDetails(sRoomName, sRoomNo);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

BOOL CVaxServerExCOM::GetRoomAll(CStringList* pRoomList)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	BOOL bResult = m_pConfRoom->GetRoomAll(pRoomList);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL CVaxServerExCOM::GetUserRouteInfo(LPCTSTR sUserName, LPCTSTR sUserDialedNo, CString& sDialNo, LONG& lToPeerType, CString& sToPeerName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	BOOL bResult = m_pRouteCallSIP->GetUserRouteInfo(sUserName, sUserDialedNo, sDialNo, lToPeerType, sToPeerName);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

BOOL CVaxServerExCOM::GetLineRouteInfo(LPCTSTR sLineName, LONG& lToPeerType, CString& sToPeerName, BOOL& bRejected, CString& sRoomName, CString& sQueueName, CString& sRingGroupName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	BOOL bResult = m_pRouteCallSIP->GetLineRouteInfo(sLineName, lToPeerType, sToPeerName, bRejected, sRoomName, sQueueName, sRingGroupName);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

BOOL CVaxServerExCOM::GetDirectProxyRouteInfo(LPCTSTR sProxyName, LONG& lToPeerType, CString& sToPeerName, BOOL& bRejected, CString& sRoomName, CString& sQueueName, CString& sRingGroupName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	BOOL bResult = m_pRouteCallSIP->GetDirectProxyRouteInfo(sProxyName, lToPeerType, sToPeerName, bRejected, sRoomName, sQueueName, sRingGroupName);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL CVaxServerExCOM::AddUserRoute(LPCTSTR sUserName, LPCTSTR sUserPrefix, LPCTSTR sDialPrefix, int nToPeerType, LPCTSTR sToPeerName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	BOOL bResult = m_pRouteCallSIP->AddUserRoute(sUserName, sUserPrefix, sDialPrefix, nToPeerType, sToPeerName);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

void CVaxServerExCOM::RemoveUserRoute(LPCTSTR sUserName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	m_pRouteCallSIP->RemoveUserRoute(sUserName);
	::LeaveCriticalSection(&m_lCriticalSection);
}

BOOL CVaxServerExCOM::AddLineRoute(LPCTSTR sLineName, int nToPeerType, LPCTSTR sToPeerName, BOOL bRejected, LPCTSTR sRoomName, LPCTSTR sQueueName, LPCTSTR sRingGroupName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	BOOL bResult = m_pRouteCallSIP->AddLineRoute(sLineName, nToPeerType, sToPeerName, bRejected, sRoomName, sQueueName, sRingGroupName);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

void CVaxServerExCOM::RemoveLineRoute(LPCTSTR sLineName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	m_pRouteCallSIP->RemoveLineRoute(sLineName);
	::LeaveCriticalSection(&m_lCriticalSection);
}

BOOL CVaxServerExCOM::AddDirectProxyRoute(LPCTSTR sProxyName, int nToPeerType, LPCTSTR sToPeerName, BOOL bRejected, LPCTSTR sRoomName, LPCTSTR sQueueName, LPCTSTR sRingGroupName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	BOOL bResult = m_pRouteCallSIP->AddDirectProxyRoute(sProxyName, nToPeerType, sToPeerName, bRejected, sRoomName, sQueueName, sRingGroupName);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

void CVaxServerExCOM::RemoveDirectProxyRoute(LPCTSTR sProxyName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	m_pRouteCallSIP->RemoveDirectProxyRoute(sProxyName);
	::LeaveCriticalSection(&m_lCriticalSection);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

BOOL CVaxServerExCOM::LoadDialToneMusic(LPCTSTR sFileName)
{
	::EnterCriticalSection(&m_lCriticalSection);

	UnLoadDialToneMusic();
	
	m_lDialToneId = CVaxServerCOM::LoadWaveFile(sFileName);
	BOOL bResult = (m_lDialToneId != -1);
	
	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

void CVaxServerExCOM::UnLoadDialToneMusic()
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	if(m_lDialToneId != -1)
	{
		CVaxServerCOM::UnLoadWaveFile(m_lDialToneId);
		m_lDialToneId = -1;
	}

	::LeaveCriticalSection(&m_lCriticalSection);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

BOOL CVaxServerExCOM::LoadMusicCallPark(LPCTSTR sFileName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	UnLoadMusicCallPark();
	
	m_lMusicCallParkId = CVaxServerCOM::LoadWaveFile(sFileName);
	BOOL bResult = (m_lMusicCallParkId != -1);
	
	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

void CVaxServerExCOM::UnLoadMusicCallPark()
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	if(m_lMusicCallParkId != -1)
	{
		CVaxServerCOM::UnLoadWaveFile(m_lMusicCallParkId);
		m_lMusicCallParkId = -1;
	}

	::LeaveCriticalSection(&m_lCriticalSection);
}

LONG CVaxServerExCOM::GetCallParkMusicId()
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	LONG lMusicCallParkId = m_lMusicCallParkId;

	::LeaveCriticalSection(&m_lCriticalSection);
	return lMusicCallParkId;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

LONG CVaxServerExCOM::GetCallCount()
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	LONG lTotalCallCount = CVaxServerCOM::GetCallCount();
	
	::LeaveCriticalSection(&m_lCriticalSection);
	return lTotalCallCount;
}

UINT CVaxServerExCOM::GetRegisteredUserCount()
{
	::EnterCriticalSection(&m_lCriticalSection);

	DWORD dwUserCount = m_pRegUserList->GetRegUserCount();

	::LeaveCriticalSection(&m_lCriticalSection);
	return (UINT) dwUserCount;
}

BOOL CVaxServerExCOM::IsRoomNoExist(LPCTSTR sRoomNo)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	BOOL bResult = m_pConfRoom->IsRoomNoExists(sRoomNo);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

CString CVaxServerExCOM::GetRoomName(LPCTSTR sRoomNo)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	CString sResult = m_pConfRoom->GetRoomName(sRoomNo);

	::LeaveCriticalSection(&m_lCriticalSection);
	return sResult;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::DeleteCallSessionDataAll()
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	if(!CVaxServerExCOM::TrvsCallSessionStart())
	{
		::LeaveCriticalSection(&m_lCriticalSection);
		return;
	}

	for(int nCount = 0; nCount < CVaxServerCOM::GetCallSessionCount(); nCount++)
	{
		LONG lSessionId = CVaxServerCOM::TrvsCallSessionNext();
		if(lSessionId == -1) continue;

		CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(lSessionId);
		if(pCallSession == NULL) continue;

		delete pCallSession;
	}

	::LeaveCriticalSection(&m_lCriticalSection);
}

void CVaxServerExCOM::DeleteUserDataAll()
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	if(!CVaxServerExCOM::TrvsUserStart())
	{
		::LeaveCriticalSection(&m_lCriticalSection);
		return;
	}

	for(int nCount = 0; nCount < CVaxServerCOM::GetUserCount(); nCount++)
	{
		CString sUserName = CVaxServerCOM::TrvsUserNext();
		if(sUserName.IsEmpty()) continue;

		CVaxUserData* pUser = (CVaxUserData*) CVaxServerCOM::GetUserData(sUserName);
		if(pUser == NULL) continue;

		delete pUser;
	}

	::LeaveCriticalSection(&m_lCriticalSection);
}

void CVaxServerExCOM::DeleteLineDataAll()
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	if(!CVaxServerExCOM::TrvsLineStart())
	{
		::LeaveCriticalSection(&m_lCriticalSection);
		return;
	}

	for(int nCount = 0; nCount < CVaxServerCOM::GetLineCount(); nCount++)
	{
		CString sLineName = CVaxServerCOM::TrvsLineNext();
		if(sLineName.IsEmpty()) continue;

		CVaxLine* pLine = (CVaxLine*) CVaxServerCOM::GetLineData(sLineName);
		if(pLine == NULL) continue;

		delete pLine;
	}

	::LeaveCriticalSection(&m_lCriticalSection);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::GetFromPeerRouteinfo(LONG lFromPeerType, LPCTSTR sDialNo, LPCTSTR sFromPeerName, CString& sToPeerName, CString& sDialToNo, CString& sRoomName, CString& sRingGroupName, CString& sQueueName, LONG& lToPeerType,  BOOL& bRejected)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	if(lFromPeerType == VAX_PEER_TYPE_USER)
	{
		if(IsRoomNoExist(sDialNo))
			sRoomName = GetRoomName(sDialNo);
		else
			GetUserRouteInfo(sFromPeerName, sDialNo, sDialToNo, lToPeerType, sToPeerName);
	}
	else if(lFromPeerType == VAX_PEER_TYPE_LINE)
	{
		GetLineRouteInfo(sFromPeerName, lToPeerType, sToPeerName, bRejected, sRoomName, sQueueName, sRingGroupName);
		sDialToNo = sToPeerName;
	}
	else if(lFromPeerType == VAX_PEER_TYPE_DIRECT_PROXY)
	{
		GetDirectProxyRouteInfo(sFromPeerName, lToPeerType, sToPeerName, bRejected, sRoomName, sQueueName, sRingGroupName);
		sDialToNo = sToPeerName;
	}

	::LeaveCriticalSection(&m_lCriticalSection);
}

/////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////// RingGroup ////////////////////////////////////////

BOOL CVaxServerExCOM::AddRingGroup(LPCTSTR sRingGroupName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	if(!m_pRingGroupList->AddRingGroup(sRingGroupName))
	{
		::LeaveCriticalSection(&m_lCriticalSection);
		return FALSE;
	}

	BOOL bResult = CVaxServerCOM::AddRingGroup(sRingGroupName);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

void CVaxServerExCOM::RemoveRingGroup(LPCTSTR sRingGroupName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	m_pRingGroupList->RemoveRingGroup(sRingGroupName);
	CVaxServerCOM::RemoveRingGroup(sRingGroupName);

	::LeaveCriticalSection(&m_lCriticalSection);
}

BOOL CVaxServerExCOM::GetRingGroupDetails(LPCTSTR sRingGroupName, int& nProcessMode, int& nProcessType, CStringList& sListAgentName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	BOOL bResult = m_pRingGroupList->GetRingGroupDetails(sRingGroupName, nProcessMode, nProcessType, sListAgentName);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

BOOL CVaxServerExCOM::AddRingGroupAgent(LPCTSTR sRingGroupName, LPCTSTR sUserName)
{
	::EnterCriticalSection(&m_lCriticalSection);

	if(!m_pRingGroupList->AddRingGroupAgent(sRingGroupName, sUserName))
	{
		::LeaveCriticalSection(&m_lCriticalSection);
		return FALSE;
	}
	
	BOOL bResult = CVaxServerCOM::AddRingGroupAgent(sRingGroupName, sUserName);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

void CVaxServerExCOM::RemoveRingGroupAgent(LPCTSTR sRingGroupName, LPCTSTR sUserName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	m_pRingGroupList->RemoveRingGroupAgent(sRingGroupName, sUserName);
	CVaxServerCOM::RemoveRingGroupAgent(sRingGroupName, sUserName);

	::LeaveCriticalSection(&m_lCriticalSection);
}

void CVaxServerExCOM::SetRingGroupProcessMode(LPCTSTR sRingGroupName, int nProcessMode, int nProcessType)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	m_pRingGroupList->SetRingGroupProcessMode(sRingGroupName, nProcessMode, nProcessType);
	CVaxServerCOM::SetRingGroupProcessMode(sRingGroupName, nProcessMode, nProcessType);

	::LeaveCriticalSection(&m_lCriticalSection);
}

BOOL CVaxServerExCOM::SetRingGroupAgentPriority(LPCTSTR sRingGroupName, LPCTSTR sAgentName, int nPriority)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	BOOL bResult = CVaxServerCOM::SetRingGroupAgentPriority(sRingGroupName, sAgentName, nPriority);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

BOOL CVaxServerExCOM::AddCallSessionToRingGroup(LPCTSTR sRingGroupName, LPCTSTR sCallerName, LPCTSTR sCallerId, LONG lSessionId)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	BOOL bResult = CVaxServerCOM::AddCallSessionToRingGroup(sRingGroupName, sCallerName, sCallerId, lSessionId);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

/////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////  Queue //////////////////////////////////////////

BOOL CVaxServerExCOM::AddQueue(LPCTSTR sQueueName, LPCTSTR sQueueMusicFile, LPCTSTR sQueueDialToneFile)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	int nQueueMusicId = CVaxServerCOM::LoadWaveFile(sQueueMusicFile);
	int nQueueDialToneId = CVaxServerCOM::LoadWaveFile(sQueueDialToneFile);
	
	if(!m_pQueueList->AddQueue(sQueueName, sQueueMusicFile, nQueueMusicId, sQueueDialToneFile, nQueueDialToneId))
	{
		CVaxServerCOM::UnLoadWaveFile(nQueueMusicId);
		CVaxServerCOM::UnLoadWaveFile(nQueueDialToneId);

		::LeaveCriticalSection(&m_lCriticalSection);
		return FALSE;
	}

	if(!CVaxServerCOM::AddQueue(sQueueName))
	{
		CVaxServerCOM::UnLoadWaveFile(nQueueMusicId);
		CVaxServerCOM::UnLoadWaveFile(nQueueDialToneId);

		m_pQueueList->RemoveQueue(sQueueName);

		::LeaveCriticalSection(&m_lCriticalSection);
		return FALSE;
	}

	::LeaveCriticalSection(&m_lCriticalSection);
	return TRUE;
}

void CVaxServerExCOM::RemoveQueue(LPCTSTR sQueueName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	int nQueueMusicId = m_pQueueList->GetQueueMusicId(sQueueName);
	CVaxServerCOM::UnLoadWaveFile(nQueueMusicId);

	int nQueueDialToneId = m_pQueueList->GetQueueDialToneId(sQueueName);
	CVaxServerCOM::UnLoadWaveFile(nQueueDialToneId);

	m_pQueueList->RemoveQueue(sQueueName);
	CVaxServerCOM::RemoveQueue(sQueueName);

	::LeaveCriticalSection(&m_lCriticalSection);
}

int CVaxServerExCOM::GetQueueMusicId(LPCTSTR sQueueName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	int nResult = m_pQueueList->GetQueueMusicId(sQueueName);

	::LeaveCriticalSection(&m_lCriticalSection);
	return nResult;
}

int CVaxServerExCOM::GetQueueDialToneId(LPCTSTR sQueueName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	int nResult = m_pQueueList->GetQueueDialToneId(sQueueName);

	::LeaveCriticalSection(&m_lCriticalSection);
	return nResult;
}

BOOL CVaxServerExCOM::AddQueueAgent(LPCTSTR sQueueName, LPCTSTR sAgentName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	if(!m_pQueueList->AddQueueAgent(sQueueName, sAgentName))
	{
		::LeaveCriticalSection(&m_lCriticalSection);
		return FALSE;
	}

	BOOL bResult = CVaxServerCOM::AddQueueAgent(sQueueName, sAgentName);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

void CVaxServerExCOM::RemoveQueueAgent(LPCTSTR sQueueName, LPCTSTR sAgentName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	m_pQueueList->RemoveQueueAgent(sQueueName, sAgentName);
	CVaxServerCOM::RemoveQueueAgent(sQueueName, sAgentName);

	::LeaveCriticalSection(&m_lCriticalSection);
}

void CVaxServerExCOM::SetQueueAgentPriority(LPCTSTR sQueueName, LPCTSTR sAgentName, LONG lPriority)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	CVaxServerCOM::SetQueueAgentPriority(sQueueName, sAgentName, lPriority);
	::LeaveCriticalSection(&m_lCriticalSection);
}

void CVaxServerExCOM::SetQueueProcessMode(LPCTSTR sQueueName, int nProcessMode, int nProcessType)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	m_pQueueList->SetQueueProcessMode(sQueueName, nProcessMode, nProcessType);
	CVaxServerCOM::SetQueueProcessMode(sQueueName, nProcessMode, nProcessType);

	::LeaveCriticalSection(&m_lCriticalSection);
}

/////////////////////////////////////////////////////////////////////////////////////
////////////////////////////      CallPickUp       //////////////////////////////////

BOOL CVaxServerExCOM::AddCallPickUpGroup(LPCTSTR sPickupGroupName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	if(!m_pCallPickUpList->AddCallPickUpGroup(sPickupGroupName))
	{
		::LeaveCriticalSection(&m_lCriticalSection);
		return FALSE;
	}

	BOOL bResult = CVaxServerCOM::AddCallPickUpGroup(sPickupGroupName);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

void CVaxServerExCOM::RemoveCallPickUpGroup(LPCTSTR sPickupGroupName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	m_pCallPickUpList->RemoveCallPickUpGroup(sPickupGroupName);
	CVaxServerCOM::RemoveCallPickUpGroup(sPickupGroupName);

	::LeaveCriticalSection(&m_lCriticalSection);
}

BOOL CVaxServerExCOM::AddCallPickUpGroupMember(LPCTSTR sPickupGroupName, LPCTSTR sMemberName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	if(!m_pCallPickUpList->AddCallPickUpGroupMember(sPickupGroupName, sMemberName))
	{
		::LeaveCriticalSection(&m_lCriticalSection);
		return FALSE;
	}

	BOOL bResult = CVaxServerCOM::AddCallPickUpGroupMember(sPickupGroupName, sMemberName);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

void CVaxServerExCOM::RemoveCallPickUpGroupMember(LPCTSTR sPickupGroupName, LPCTSTR sMemberName)
{
	::EnterCriticalSection(&m_lCriticalSection);

	m_pCallPickUpList->RemoveCallPickUpGroupMember(sPickupGroupName, sMemberName);
	CVaxServerCOM::RemoveCallPickUpGroupMember(sPickupGroupName, sMemberName);

	::LeaveCriticalSection(&m_lCriticalSection);
}

BOOL CVaxServerExCOM::PickUpCall(LONG lSessionId,  LPCTSTR sMemberName)
{
	::EnterCriticalSection(&m_lCriticalSection);
	
	BOOL bResult = CVaxServerCOM::PickUpCall(lSessionId, sMemberName);

	::LeaveCriticalSection(&m_lCriticalSection);
	return bResult;
}

BOOL CVaxServerExCOM::VoiceSessionLost(BOOL bEnable, LONG lTimeout)
{
	::EnterCriticalSection(&m_lCriticalSection);
	BOOL bResult = CVaxServerCOM::VoiceSessionLost(bEnable, lTimeout);
	::LeaveCriticalSection(&m_lCriticalSection);

	return bResult;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnRegisterUser(LPCTSTR sUserLogin, LPCTSTR sDomain, LPCTSTR sUserAgent, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId)
{
	if(CVaxServerCOM::GetUserData(sUserLogin) == NULL)
	{
		CVaxServerCOM::RejectRegister(nRegId, 404, "Not Found");
		return;
	}

	CVaxRegUser* pRegUser = m_pRegUserList->AddRegUserData(sUserLogin);

	if(pRegUser == NULL)
	{
		CVaxServerCOM::RejectRegister(nRegId, 404, "Not Found");
		return;
	}

	pRegUser->OnRegisterUser(CString(sUserLogin), CString(sDomain), CString(sUserAgent), CString(sFromIP), nFromPort, nRegId);

	CString sStatus;
	sStatus.Format("%s %s", sUserLogin, USER_REG_TRYING);

	GenStatusLog(sStatus);
}

void CVaxServerExCOM::OnRegisterUserSuccess(LPCTSTR sUserLogin, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId)
{
	CVaxRegUser* pRegUser =  m_pRegUserList->GetRegUserData(sUserLogin);
	if(pRegUser == NULL) return;

	pRegUser->OnRegisterUserSuccess(sUserLogin, sFromIP, nFromPort, nRegId);

	CString sStatus;
	sStatus.Format("%s %s", sUserLogin, USER_REG_SUCCESS);

	GenStatusLog(sStatus);
}

void CVaxServerExCOM::OnRegisterUserFailed(LPCTSTR sUserLogin, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId)
{
	CVaxRegUser* pRegUser =  m_pRegUserList->GetRegUserData(sUserLogin);
	if(pRegUser == NULL) return;

	pRegUser->OnRegisterUserFailed(sUserLogin, sFromIP, nFromPort, nRegId);
	m_pRegUserList->RemoveRegUserData(sUserLogin);

	CString sStatus;
	sStatus.Format("%s %s", sUserLogin, USER_REG_FAILED);

	GenStatusLog(sStatus);
}

void CVaxServerExCOM::OnUnRegisterUser(LPCTSTR sUserLogin)
{
	CVaxRegUser* pRegUser =  m_pRegUserList->GetRegUserData(sUserLogin);
	
	if(pRegUser == NULL)
		return;
			
	pRegUser->OnUnRegisterUser(sUserLogin);
	m_pRegUserList->RemoveRegUserData(sUserLogin);

	CString sStatus;
	sStatus.Format("%s %s", sUserLogin, USER_UNREGISTER);

	GenStatusLog(sStatus);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnIncomingCall(LONG nSessionId, LPCTSTR sCallerName, LPCTSTR sCallerId, LPCTSTR sDialNo, LONG nFromPeerType, LPCTSTR sFromPeerName, LPCTSTR sUserAgent, LPCTSTR sFromIP, LONG nFromPort)
{
	CString sToPeerName , sDialToNo;
	CString sRingGroupName, sQueueName, sRoomName;
	
	LONG lToPeerType = -1;
	BOOL bRejected = FALSE;

	GetFromPeerRouteinfo(nFromPeerType, sDialNo, sFromPeerName, sToPeerName, sDialToNo, sRoomName, sRingGroupName, sQueueName, lToPeerType, bRejected);

	if(bRejected)
	{
		CVaxServerCOM::RejectCallSession(nSessionId, 404, "Not Found");
		return;
	}

	DialToneToCallSession(TRUE, nSessionId);

	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnIncomingCall(nSessionId, sCallerName, sCallerId, sDialNo, sRoomName, sQueueName, sRingGroupName, nFromPeerType, sFromPeerName, lToPeerType, sToPeerName, sUserAgent, sFromIP, nFromPort);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnCallSessionCreated(LONG nSessionId, LONG nReasonCode)
{
	CVaxCallSession* pCallSession = new CVaxCallSession(this); 
	CVaxServerCOM::SetCallSessionData(nSessionId, (HANDLE) pCallSession);
}

void CVaxServerExCOM::OnCallSessionClosed(LONG nSessionId, LONG nChannelId, LONG nReasonCode)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	delete pCallSession;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnCallSessionOnHold(LONG nSessionId, LONG nChannelId)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionOnHold(nSessionId, nChannelId);
}

void CVaxServerExCOM::OnCallSessionOffHold(LONG nSessionId, LONG nChannelId)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionOffHold(nSessionId, nChannelId);
}

///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnCallSessionTransferBlind(LONG nTransfererSessionId, LONG nTransfererChannelId, LPCTSTR sTransferer, LPCTSTR sTransferee, LPCTSTR sTransferTo)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nTransfererSessionId);
	pCallSession->OnCallSessionTransferBlind(nTransfererSessionId, nTransfererChannelId, sTransferer, sTransferee, sTransferTo);
}

void CVaxServerExCOM::OnCallSessionTransferConsult(LONG nTransfererSessionId, LONG nTransfereeSessionId, LONG nTransferToSessionId, LONG nTransfererChannelId, LONG nTransfereeChannelId, LONG nTransferToChannelId, LPCTSTR sTransferer, LPCTSTR sTransferee, LPCTSTR sTransferTo)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nTransfererSessionId);
	pCallSession->OnCallSessionTransferConsult(nTransfererSessionId, nTransfererChannelId, nTransferToSessionId, nTransferToChannelId, sTransferer, sTransferee, sTransferTo);
}

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnChatMessageText(LONG nChatMsgId, LPCTSTR sMsgFrom, LPCTSTR sMsgTo, LPCTSTR sMsgText, LONG nFromPeerType, LPCTSTR sFromPeerName, LPCTSTR sFromIP, LONG nFromPort)
{
	m_pChat->OnChatMessageText(nChatMsgId, sMsgFrom, sMsgTo, sMsgText, nFromPeerType, sFromPeerName, sFromIP, nFromPort);
}

void CVaxServerExCOM::OnChatMessageTyping(LONG nChatMsgId, LPCTSTR sMsgFrom, LPCTSTR sMsgTo, BOOL IsTypingStart, LONG nFromPeerType, LPCTSTR sFromPeerName, LPCTSTR sFromIP, LONG nFromPort)
{
	m_pChat->OnChatMessageTyping(nChatMsgId, sMsgFrom, sMsgTo, IsTypingStart, nFromPeerType, sFromPeerName, sFromIP, nFromPort);
}

void CVaxServerExCOM::OnChatStatusSubscribe(LONG nSubscrbId, LPCTSTR sMsgFrom, LPCTSTR sMsgTo, LONG nFromPeerType, LPCTSTR sFromPeerName, LPCTSTR sFromIP, LONG nFromPort)
{
	m_pChat->OnChatStatusSubscribe(nSubscrbId, sMsgFrom, sMsgTo, nFromPeerType, sFromPeerName, sFromIP, nFromPort);
}

void CVaxServerExCOM::OnChatMessageSuccess(LONG nChatMsgId)
{
	m_pChat->OnChatMessageSuccess(nChatMsgId);
}

void CVaxServerExCOM::OnChatMessageFailed(LONG nChatMsgId, LONG nStatusId, LPCTSTR sReasonPhrase)
{
	m_pChat->OnChatMessageFailed(nChatMsgId, nStatusId, sReasonPhrase);
}

void CVaxServerExCOM::OnChatMessageTimeout(LONG nChatMsgId)
{
	m_pChat->OnChatMessageTimeout(nChatMsgId);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnLineRegisterTrying(LPCTSTR sLineName)
{
	CVaxLine* pLine = (CVaxLine*) CVaxServerCOM::GetLineData(sLineName);
	pLine->OnLineRegisterTrying(sLineName);

	CString sStatus;
	sStatus.Format("'%s' %s", sLineName, LINE_REG_TRYING);

	GenStatusLog(sStatus);
}

void CVaxServerExCOM::OnLineRegisterFailed(LPCTSTR sLineName, LONG nStatusCode, LPCTSTR sReasonPhrase)
{
	CVaxLine* pLine = (CVaxLine*) CVaxServerCOM::GetLineData(sLineName);
	pLine->OnLineRegisterFailed(sLineName, nStatusCode, sReasonPhrase);

	CString sStatus;
	sStatus.Format("'%s' Register Failed: %s", sLineName, sReasonPhrase);

	GenStatusLog(sStatus);
}

void CVaxServerExCOM::OnLineRegisterSuccess(LPCTSTR sLineName)
{
	CVaxLine* pLine = (CVaxLine*) CVaxServerCOM::GetLineData(sLineName);
	pLine->OnLineRegisterSuccess(sLineName);

	CString sStatus;
	sStatus.Format("'%s' %s", sLineName, LINE_REG_SUCCESS);

	GenStatusLog(sStatus);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnDetectedDigitDTMF(LONG nSessionId, LONG nChannelId, LPCTSTR sDigitDTMF)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnDetectedDigitDTMF(nSessionId, nChannelId, sDigitDTMF);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::GenStatusLog(LPCTSTR sStatus)
{
	OnStatusLog(sStatus);
}

void CVaxServerExCOM::OnVaxErrorLog(LPCTSTR sFuncName, LONG nErrorCode, LPCTSTR sErrorMsg)
{
	CString sMsg;
	sMsg.Format("[Method: %s] [Error Msg: %s] [Code: %d]", sFuncName, sErrorMsg, nErrorCode);

	OnErrorLog(sMsg);
}

void CVaxServerExCOM::OnInitialize()
{

}

void CVaxServerExCOM::OnStatusLog(LPCTSTR sMsg)
{

}

void CVaxServerExCOM::OnErrorLog(LPCTSTR sMsg)
{

}

void CVaxServerExCOM::OnCallSessionTransfering(LONG nSessionId, LONG nTransferType, LONG nChannelId, LONG nStatusCode, LPCTSTR sReasonPhrase)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionTransfering(nSessionId, nTransferType, nChannelId, nStatusCode, sReasonPhrase);
}

void CVaxServerExCOM::OnCallSessionTransferFailed(LONG nSessionId, LONG nTransferType, LONG nChannelId, LONG nStatusCode, LPCTSTR sReasonPhrase)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionTransferFailed(nSessionId, nTransferType, nChannelId, nStatusCode, sReasonPhrase);
}

void CVaxServerExCOM::OnCallSessionTransferTimeout(LONG nSessionId, LONG nTransferType, LONG nChannelId)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionTransferTimeout(nSessionId, nTransferType, nChannelId);
}

void CVaxServerExCOM::OnCallSessionTransfered(LONG nSessionId, LONG nTransferType)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionTransfered(nSessionId, nTransferType);
}

/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnOutgoingDiagnosticLog(LPCTSTR sMsgSIP, LPCTSTR sToIP, LONG nToPort)
{

}

void CVaxServerExCOM::OnIncomingDiagnosticLog(LPCTSTR sMsgSIP, LPCTSTR sFromIP, LONG nFromPort)
{

}

/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnVaxTeleTick(LONG nTickId)
{

}

/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnSendTimeoutVM(LONG nMsgIdVM)
{

}

void CVaxServerExCOM::OnSendSuccessVM(LONG nMsgIdVM)
{

}

/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnCallSessionDialToneStart(LONG nSessionId, LONG nChannelId, LONG nWaveId)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionDialToneStart(nSessionId, nChannelId, nWaveId);
}

void CVaxServerExCOM::OnCallSessionDialToneStop(LONG nSessionId, LONG nChannelId, LONG nWaveId)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionDialToneStop(nSessionId, nChannelId, nWaveId);
}

/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnCallSessionPlayWaveDone(LONG nSessionId, LONG nChannelId, LONG nWaveId)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionPlayWaveDone(nSessionId, nChannelId, nWaveId);
}

void CVaxServerExCOM::OnConferenceRoomPlayWaveDone(LPCTSTR sRoomName, LONG nWaveId)
{

}

/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnCallSessionLost(LONG nSessionId, LONG nChannelId)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionLost(nSessionId, nChannelId);
}

void CVaxServerExCOM::OnCallSessionHangup(LONG nSessionId, LONG nChannelId)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionHangup(nSessionId, nChannelId);
}

void CVaxServerExCOM::OnCallSessionTimeout(LONG nSessionId, LONG nChannelId)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionTimeout(nSessionId, nChannelId);
}

void CVaxServerExCOM::OnCallSessionCancelled(LONG nSessionId)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionCancelled(nSessionId);
}

void CVaxServerExCOM::OnCallSessionDetectedAMD(LONG nSessionId, LONG nChannelId, BOOL bIsHuman)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionDetectedAMD(nSessionId, nChannelId, bIsHuman);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnCallSessionRecordedPCM(LONG nSessionId, LONG nChannelId, LONGLONG nDataPCM, LONG nSizePCM, LONG nTypePCM)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionRecordedPCM(nSessionId, nChannelId, (char*) nDataPCM, nSizePCM); 
}

void CVaxServerExCOM::OnConferenceRoomRecordedPCM(LPCTSTR sRoomName, LONGLONG nDataPCM, LONG nSizePCM, LONG nTypePCM)
{

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnBusyLampSubscribe(LONG nSubScrbId, LPCTSTR sUserName, LPCTSTR sToUserName, LPCTSTR sFromIP, LONG nFromPort)
{
	CVaxServerCOM::BusyLampSubscribeAccept(nSubScrbId, TRUE, 30);
}

void CVaxServerExCOM::OnBusyLampUnSubscribe(LONG nSubScrbId, LPCTSTR sUserName, LPCTSTR sToUserName, LPCTSTR sFromIP, LONG nFromPort)
{

}

void CVaxServerExCOM::OnBusyLampSubscribeSuccess(LONG nSubScrbId, LPCTSTR sUserName)
{

}

void CVaxServerExCOM::OnBusyLampSubscribeFailed(LONG nSubScrbId, LPCTSTR sUserName)
{

}

void CVaxServerExCOM::OnBusyLampSendStatus(LPCTSTR sFromUserName, LPCTSTR sToUserName, LONG nStateId)
{
	CVaxServerCOM::BusyLampSendStatus(sFromUserName, sToUserName, nStateId);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CVaxServerExCOM::OnVaxLicenseChannel(LONG nTotalChannels, LONG nUsedChannels)
{
}

void CVaxServerExCOM::OnCallSessionConnecting(LONG nSessionId, LONG nChannelId, LONG nStatusCode, LPCTSTR sReasonPhrase)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionConnecting(nSessionId, nChannelId, nStatusCode, sReasonPhrase);
}

void CVaxServerExCOM::OnCallSessionFailed(LONG nSessionId, LONG nChannelId, LONG nStatusCode, LPCTSTR sReasonPhrase)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionFailed(nSessionId, nChannelId, nStatusCode, sReasonPhrase);
}

void CVaxServerExCOM::OnCallSessionConnected(LONG nSessionId)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnCallSessionConnected(nSessionId);
}

void CVaxServerExCOM::OnLineUnRegisterTrying(LPCTSTR sLineName)
{
}

void CVaxServerExCOM::OnLineUnRegisterFailed(LPCTSTR sLineName, LONG nStatusCode, LPCTSTR sReasonPhrase)
{
}

void CVaxServerExCOM::OnLineUnRegisterSuccess(LPCTSTR sLineName)
{
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void  CVaxServerExCOM::OnAddCallSessionToQueueSuccess(LPCTSTR sQueueName, LONG nSessionId)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnAddCallSessionToQueueSuccess(sQueueName, nSessionId);
}

void  CVaxServerExCOM::OnAddCallSessionToQueueFailed(LPCTSTR sQueueName, LONG nSessionId)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnAddCallSessionToQueueFailed(sQueueName, nSessionId);
}

void  CVaxServerExCOM::OnQueuePlayWaveStart(LONG nEventType, LPCTSTR sQueueName, LONG nSessionId)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnQueuePlayWaveStart(nEventType, sQueueName, nSessionId);

}

void  CVaxServerExCOM::OnQueuePlayWaveStop(LONG nEventType, LPCTSTR sQueueName, LONG nSessionId)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnQueuePlayWaveStop(nEventType, sQueueName, nSessionId);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void  CVaxServerExCOM::OnAddCallSessionToRingGroupSuccess(LPCTSTR sGroupName, LONG nSessionId)
{
	CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	pCallSession->OnAddCallSessionToRingGroupSuccess(nSessionId, sGroupName);
}

void  CVaxServerExCOM::OnAddCallSessionToRingGroupFailed(LPCTSTR sGroupName, LONG nSessionId)
{
	//CVaxCallSession* pCallSession = (CVaxCallSession*) CVaxServerCOM::GetCallSessionData(nSessionId);
	//pCallSession->OnAddCallSessionToRingGroupFailed(nSessionId, sGroupName);
}
