// AddDirectProxyDlg.cpp : implementation file
//

#include "stdafx.h"
#include "VaxTeleServerApp.h"
#include "AddDirectProxyDlg.h"
#include "afxDialogEx.h"
#include "Codec.h"
#include "VaxServerSIP.h"

// CAddDirectProxyDlg dialog

IMPLEMENT_DYNAMIC(CAddDirectProxyDlg, CPropertyPage)

CAddDirectProxyDlg::CAddDirectProxyDlg(CVaxServerSIP* pVaxTeleSIP) : CPropertyPage(CAddDirectProxyDlg::IDD)
{
	m_pVaxTeleSIP = pVaxTeleSIP;
	m_nProxyPort = 5060;

	m_sProxyAuthUser = _T("");
	m_sProxyPassword = _T("");
	m_sProxyRealm = _T("");
	m_sProxySIP = _T("");
	m_nRadioCtrlInboundCall = 0;
	m_sProxyName = _T("");
	m_sConnectToLogin = _T("");
	m_sRoomName = _T("");
}

CAddDirectProxyDlg::~CAddDirectProxyDlg()
{
}

void CAddDirectProxyDlg::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LIST_PROXY_DISABLE_CODEC, m_ctrlDisableCodecList);
	DDX_Control(pDX, IDC_LIST_PROXY_ENABLE_CODEC, m_ctrlEnableCodecList);
	DDX_Control(pDX, IDC_COMBO_PROXY_CONNECT_USER, m_ctrlComboConnectToLogin);
	DDX_Control(pDX, IDC_COMBO_PROXY, m_ctrlComboProxy);
	DDX_Text(pDX, IDC_EDIT_PROXY_AUTH_USER, m_sProxyAuthUser);
	DDX_Text(pDX, IDC_EDIT_PROXY_PASSWORD, m_sProxyPassword);
	DDX_Text(pDX, IDC_EDIT_PROXY_DOMAIN_REALM, m_sProxyRealm);
	DDX_Text(pDX, IDC_EDIT_PROXY_SIP_PROXY, m_sProxySIP);
	DDX_Radio(pDX, IDC_RADIO_PROXY_INBOUND_REJECT, m_nRadioCtrlInboundCall);
	DDX_Text(pDX, IDC_EDIT_PROXY_PORT, m_nProxyPort);
	DDX_CBString(pDX, IDC_COMBO_PROXY, m_sProxyName);
	DDX_Control(pDX, IDC_COMBO_PROXY_CONNECT_ROOM, m_ctrlComboConnectToRoom);
	DDX_CBString(pDX, IDC_COMBO_PROXY_CONNECT_USER, m_sConnectToLogin);
	DDX_CBString(pDX, IDC_COMBO_PROXY_CONNECT_ROOM, m_sRoomName);
}


BEGIN_MESSAGE_MAP(CAddDirectProxyDlg, CPropertyPage)
	ON_BN_CLICKED(IDC_BUTTON_PROXY_DISABLE_CODEC, &CAddDirectProxyDlg::OnButtonProxyDisableCodec)
	ON_BN_CLICKED(IDC_BUTTON_PROXY_ENABLE_CODEC, &CAddDirectProxyDlg::OnButtonProxyEnableCodec)
	ON_BN_CLICKED(IDC_BUTTON_PROXY_ADD, &CAddDirectProxyDlg::OnButtonProxyAdd)
	ON_BN_CLICKED(IDC_BUTTON_PROXY_REMOVE, &CAddDirectProxyDlg::OnButtonProxyRemove)
	ON_BN_CLICKED(IDC_BUTTON_CLEAR_PROXY_DLG, &CAddDirectProxyDlg::OnButtonClearProxyDlg)
	ON_CBN_SELCHANGE(IDC_COMBO_PROXY, &CAddDirectProxyDlg::OnSelChangeComboProxy)
	ON_WM_DESTROY()
END_MESSAGE_MAP()


// CAddDirectProxyDlg message handlers

BOOL CAddDirectProxyDlg::OnSetActive()
{
	AddDirectProxies();
	
	AddUsers();
	AddRooms();
	
	ShowProxyDetails();

	return CPropertyPage::OnSetActive();
}

BOOL CAddDirectProxyDlg::OnInitDialog()
{
	CPropertyPage::OnInitDialog();

	m_ctrlComboConnectToLogin.SetWindowPos(NULL,0,0,120,150,SWP_NOMOVE|SWP_NOZORDER);
	m_ctrlComboConnectToRoom.SetWindowPos(NULL,0,0,120,150,SWP_NOMOVE|SWP_NOZORDER);

	LoadCodecs();

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CAddDirectProxyDlg::OnButtonProxyDisableCodec()
{
	int nCurSel = m_ctrlEnableCodecList.GetCurSel();
	if(nCurSel == -1) return;

	CString sCurSelText;
	m_ctrlEnableCodecList.GetText(nCurSel, sCurSelText);
	m_ctrlEnableCodecList.DeleteString(nCurSel);
	
	m_ctrlDisableCodecList.AddString(sCurSelText);
}

void CAddDirectProxyDlg::OnButtonProxyEnableCodec()
{
	int nCurSel = m_ctrlDisableCodecList.GetCurSel();
	if(nCurSel == -1) return;

	CString sCurSelText;
	
	m_ctrlDisableCodecList.GetText(nCurSel, sCurSelText);
	m_ctrlDisableCodecList.DeleteString(nCurSel);
	
	m_ctrlEnableCodecList.AddString(sCurSelText);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CAddDirectProxyDlg::OnButtonProxyAdd()
{
	UpdateData();
	
	if(!AddProxyChecks())
		return;

	if(m_ctrlComboProxy.FindStringExact(0, m_sProxyName) != -1)
	{
		if(MessageBox("Direct-Proxy already exists, do you want to continue?", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_YESNO) == IDNO)
			return;
	}
	
	///////////////////////////////////////////////////////////////////////////

	CCodec objCodec;
	CString sCodecList;
		
	for(int nCount = 0; nCount < m_ctrlEnableCodecList.GetCount(); nCount++)
	{
		CString sCodecText;
		m_ctrlEnableCodecList.GetText(nCount, sCodecText);

		sCodecList += objCodec.GetCodecNo(sCodecText);
	}

	///////////////////////////////////////////////////////////////////////////

	CString sToPeerName, sRoomName;
	BOOL bRejected = FALSE; int nToPeerType = -1;

	if(m_nRadioCtrlInboundCall == 0)
		bRejected = TRUE;

	else if(m_nRadioCtrlInboundCall == 1)
	{
		nToPeerType = VAX_PEER_TYPE_USER;
		sToPeerName = m_sConnectToLogin;
	}
	else if(m_nRadioCtrlInboundCall == 2)
	{
		sRoomName = m_sRoomName;
	}

	//////////////////////////////////////////////////////////////////////////

	m_pVaxTeleSIP->RemoveDirectProxySIP(m_sProxyName);

	if(!m_pVaxTeleSIP->AddDirectProxySIP(m_sProxyName, m_sProxyAuthUser, m_sProxyPassword, m_sProxyRealm, m_sProxySIP, m_nProxyPort, sCodecList, nToPeerType, sToPeerName, bRejected, sRoomName))
	{
		MessageBox("Direct-Proxy could not be added/updated or Please start the server first.", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_OK);
		return;
	}
	
	if(m_ctrlComboProxy.FindStringExact(0, m_sProxyName) == -1)
		m_ctrlComboProxy.AddString(m_sProxyName);

	MessageBox("Direct-Proxy added/updated successfully.", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_OK);
}

void CAddDirectProxyDlg::OnButtonProxyRemove()
{
	UpdateData();
	
	if(m_ctrlComboProxy.GetCount() <= 0)
	{
		MessageBox("Please enter a proxy name.", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_OK);
		return;
	}
	
	int nIndex = m_ctrlComboProxy.FindStringExact(0, m_sProxyName);
	if(nIndex == -1) return;

	m_pVaxTeleSIP->RemoveDirectProxySIP(m_sProxyName);
	m_ctrlComboProxy.DeleteString(nIndex);
	
	ClearDlgContent();
	
	if(m_ctrlComboProxy.GetCount() > 0)
		m_ctrlComboProxy.SetCurSel(0);
		
	ShowProxyDetails();
	
	MessageBox("Direct-Proxy removed successfully!", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_OK);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CAddDirectProxyDlg::OnButtonClearProxyDlg()
{
	ClearDlgContent();
}

void CAddDirectProxyDlg::OnSelChangeComboProxy()
{
   ShowProxyDetails();
}

void CAddDirectProxyDlg::ClearDlgContent()
{
	UpdateData(TRUE);
	
	m_ctrlComboConnectToLogin.SetCurSel(0);
	
	m_ctrlEnableCodecList.ResetContent();
	m_ctrlDisableCodecList.ResetContent();

	m_ctrlEnableCodecList.AddString(G711U_TEXT);
	m_ctrlEnableCodecList.AddString(G711A_TEXT);
	m_ctrlEnableCodecList.AddString(ILBC_TEXT);
	m_ctrlEnableCodecList.AddString(G610_TEXT);
	m_ctrlEnableCodecList.AddString(G729_TEXT);

	m_sProxyName		= "";
	m_sProxyAuthUser	= "";
	m_sProxyPassword	= "";
	m_sProxyRealm		= "";
	m_sProxySIP			= "";
	m_nProxyPort		= 5060;
	
	m_nRadioCtrlInboundCall = 0;
	
	UpdateData(FALSE);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CAddDirectProxyDlg::ShowProxyDetails()
{
	int nCurIndex = m_ctrlComboProxy.GetCurSel();
	if(nCurIndex == -1) return;

	m_ctrlComboProxy.GetLBText(nCurIndex, m_sProxyName);
	UpdateData(FALSE);
			
	CString sAuthUser, sAuthPwd; 
	CString sDomainRealm; CString sProxySIP; 
	CString sProxyPort; CString sCodecList;
	CString	sRoomName; CString sToPeerName;

	BOOL bRejected;
	LONG lProxyPort; int nToPeerType = -1;

	m_pVaxTeleSIP->GetDirectProxyDetails(m_sProxyName, sAuthUser, sAuthPwd, sDomainRealm, sProxySIP, lProxyPort, sCodecList, nToPeerType, sToPeerName, bRejected, sRoomName);
	UpdateDlgContent(m_sProxyName, sAuthUser, sAuthPwd, sDomainRealm, sProxySIP, sCodecList, lProxyPort, nToPeerType, sToPeerName, bRejected, sRoomName);
}

void CAddDirectProxyDlg::UpdateDlgContent(LPCTSTR sProxyName, LPCTSTR sAuthUser, LPCTSTR sAuthPwd, LPCTSTR sRealm, LPCTSTR sProxySIP, CString sCodecList, DWORD dwProxyPort, int nToPeerType, LPCTSTR sToPeerName, BOOL bRejected, LPCTSTR sRoomName)
{
	UpdateData();

	m_sProxyName = sProxyName;
	m_sProxyAuthUser = sAuthUser;
	m_sProxyPassword = sAuthPwd;
	m_sProxyRealm = sRealm;
	m_sProxySIP = sProxySIP;
	m_nProxyPort = dwProxyPort;

	UpdateIncomingCallSection(nToPeerType, sToPeerName, bRejected, sRoomName);
	
	CCodec objCodec;
	
	m_ctrlEnableCodecList.ResetContent();
	m_ctrlDisableCodecList.ResetContent();

	for(char cCodec = '0'; cCodec < TOTAL_NO_CODEC; cCodec++)	
		m_ctrlDisableCodecList.AddString(objCodec.GetCodecName(cCodec));
		
	for(int nCount = 0; nCount < sCodecList.GetLength(); nCount++)
	{
		CString sCodecName = objCodec.GetCodecName(sCodecList.GetAt(nCount));
		m_ctrlEnableCodecList.AddString(sCodecName);
		
		int nIndex = m_ctrlDisableCodecList.FindStringExact(0, sCodecName);
		if(nIndex == -1) continue;
		
		m_ctrlDisableCodecList.DeleteString(nIndex);
	}
			
	UpdateData(FALSE);
}

void CAddDirectProxyDlg::UpdateIncomingCallSection(int nToPeerType, LPCTSTR sToPeerName, BOOL bRejected, LPCTSTR sRoomName)
{
	if(bRejected)
		m_nRadioCtrlInboundCall = 0;

	if(nToPeerType == VAX_PEER_TYPE_USER)
	{
		m_nRadioCtrlInboundCall = 1;
		m_ctrlComboConnectToLogin.SelectString(0, sToPeerName);
	}

	if(!CString(sRoomName).IsEmpty())
	{
		m_nRadioCtrlInboundCall = 2;
		m_ctrlComboConnectToRoom.SelectString(0, sRoomName);
	}
}

void CAddDirectProxyDlg::LoadCodecs()
{
	CCodec objCodec;
	
	m_ctrlEnableCodecList.ResetContent();
	m_ctrlDisableCodecList.ResetContent();

	for(char cCodec = '0'; cCodec < TOTAL_NO_CODEC; cCodec++)	
		m_ctrlEnableCodecList.AddString(objCodec.GetCodecName(cCodec));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CAddDirectProxyDlg::AddDirectProxies()
{
	m_ctrlComboProxy.ResetContent();

	CStringList objProxyList;
	m_pVaxTeleSIP->GetDirectProxyAll(&objProxyList);
		
	while(!objProxyList.IsEmpty())
		m_ctrlComboProxy.AddString(objProxyList.RemoveHead());

	if(m_ctrlComboProxy.GetCount() > 0)
		m_ctrlComboProxy.SetCurSel(0);
}

void CAddDirectProxyDlg::AddUsers()
{
	m_ctrlComboConnectToLogin.ResetContent();

	CStringList objListUsers;
	m_pVaxTeleSIP->GetUserAll(&objListUsers);

	while(!objListUsers.IsEmpty())
		m_ctrlComboConnectToLogin.AddString(objListUsers.RemoveHead());
			
	if(m_ctrlComboConnectToLogin.GetCount() > 0)
		m_ctrlComboConnectToLogin.SetCurSel(0);
}

void CAddDirectProxyDlg::AddRooms()
{
	m_ctrlComboConnectToRoom.ResetContent();

	CStringList objRoomList;
	m_pVaxTeleSIP->GetRoomAll(&objRoomList);

	if(objRoomList.GetCount() <= 0) return;

	while(!objRoomList.IsEmpty())
		m_ctrlComboConnectToRoom.AddString(objRoomList.RemoveHead());
		
	if(m_ctrlComboConnectToRoom.GetCount() > 0)
		m_ctrlComboConnectToRoom.SetCurSel(0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL CAddDirectProxyDlg::AddProxyChecks()
{
	UpdateData();
	
	if(m_sProxyName.IsEmpty())
	{
		MessageBox("Please enter a proxy name.", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_OK);
		return FALSE;
	}
		
	if(m_ctrlEnableCodecList.GetCount() == 0)
	{
		MessageBox("Please select a codec.", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_OK);
		return FALSE;
	}
	
	return TRUE;
}

void CAddDirectProxyDlg::OnDestroy()
{
	CPropertyPage::OnDestroy();
}
